;TODO : make macros which create exception handler and jump to handler

; entry point of every exceptions.
extern HANDLER_TestException
extern HANDLER_DivideZero
extern HANDLER_Debug
extern HANDLER_NMI
extern HANDLER_BreakPoint
extern HANDLER_Overflow
extern HANDLER_BOUNDExceed
extern HANDLER_InvalidOpcode
extern HANDLER_DeviceDisable
extern HANDLER_DoubleFault
extern HANDLER_CoprocessorSegOver
extern HANDLER_InvalidTSS
extern HANDLER_SegNotPresent
extern HANDLER_StackFault
extern HANDLER_GeneralProtection
extern HANDLER_PageFault
extern HANDLER_INTELReserved
extern HANDLER_FloatingPoint
extern HANDLER_AlignCheck
extern HANDLER_MachineCheck
extern HANDLER_SIMDFault

; entry point of every IRQ.
extern HandleIRQ


; save all register, it save context of current process.
%macro	SAVE_ALL 0
	push eax
	push es
	push ds
	push ebp
	push esi
	push edi
	push edx
	push ecx
	push ebx

	; set KERNEL CODE segment
	mov ebx, 0x10
	mov ds, bx
	mov es, bx
%endmacro

; restore all register, it restore context of previous process
%macro	RESTORE_ALL 0
	pop ebx
	pop ecx
	pop edx
	pop edi
	pop esi
	pop ebp
	pop ds
	pop es
	pop eax
%endmacro



; exception handler,
; argument : 1. entry point of exception
;            2. handler of exception

%macro ExceptionNoError 2
	global %1
	align 4
%1:
	push 0x0	; no error code
	push %2		; push base of handler
	
	jmp InvokeHandler
%endmacro

%macro ExceptionWithError 2
	global %1
	align 4
%1:
	push %2

	jmp InvokeHandler
%endmacro

; external interrupt handler
; arguments : 1. entry point of interrupt
;             2. irq number
%macro HandleIRQ_macro 2
	global %1
	align 4
%1:
	SAVE_ALL

	push %2
	call HandleIRQ
	add esp, 4

	;test byte[scheduling], 0x1
	;jz no_schedule
	;call schedule

	RESTORE_ALL
	iret
%endmacro


section .text

; test handler
	global TestException
	align 4
TestException:
	pusha
	call HANDLER_TestException
	popa
	ret

; invoke real exception handler
	align 4
InvokeHandler:
	SAVE_ALL
	
	; call handler
	; address of handler is stored in stack already 
	; by ExceptionNoError or ExceptionWithError
	mov edx, [esp+36]
	call edx

	RESTORE_ALL

	add esp, 8
	iret
	
; handlers without error code
; DivideZero, Debug, NMI, BreakPoint, Overflow, BOUNDExceed, InvalidOpcode
; DeviceDisable, CoprocessorSegOver, INTELReserved, FloatingPoint, 
; MachineCheck, SIMDFault
;ExceptionNoError TestException, HANDLER_TestException
ExceptionNoError DivideZero, HANDLER_DivideZero
ExceptionNoError Debug, HANDLER_Debug
ExceptionNoError NMI, HANDLER_NMI
ExceptionNoError BreakPoint, HANDLER_BreakPoint
ExceptionNoError Overflow, HANDLER_Overflow
ExceptionNoError BOUNDExceed, HANDLER_BOUNDExceed
ExceptionNoError InvalidOpcode, HANDLER_InvalidOpcode
ExceptionNoError DeviceDisable, HANDLER_DeviceDisable
ExceptionNoError CoprocessorSegOver, HANDLER_CoprocessorSegOver
ExceptionNoError INTELReserved, HANDLER_INTELReserved
ExceptionNoError FloatingPoint, HANDLER_FloatingPoint
ExceptionNoError MachineCheck, HANDLER_MachineCheck
ExceptionNoError SIMDFault, HANDLER_SIMDFault

; handlers with error code
; DoubleFault, InvalidTSS, SegNotPresent, StackFault, GeneralProtection
; PageFault, AlignCheck
ExceptionWithError DoubleFault, HANDLER_DoubleFault
ExceptionWithError InvalidTSS, HANDLER_InvalidTSS
ExceptionWithError SegNotPresent, HANDLER_SegNotPresent
ExceptionWithError StackFault, HANDLER_StackFault
ExceptionWithError GeneralProtection, HANDLER_GeneralProtection
ExceptionWithError PageFault, HANDLER_PageFault
ExceptionWithError AlignCheck, HANDLER_AlignCheck

; this exception handler must be stored with not-used table
	global UnidentifiedINTR
	align 4
UnidentifiedINTR:
	mov edi, 0xc00b8000
	mov al, 'X'
	stosb
	halt
	iret

; define irq handlers 0~15
HandleIRQ_macro HandleIRQ_0, 0
HandleIRQ_macro HandleIRQ_1, 1
HandleIRQ_macro HandleIRQ_2, 2
HandleIRQ_macro HandleIRQ_3, 3
HandleIRQ_macro HandleIRQ_4, 4
HandleIRQ_macro HandleIRQ_5, 5
HandleIRQ_macro HandleIRQ_6, 6
HandleIRQ_macro HandleIRQ_7, 7
HandleIRQ_macro HandleIRQ_8, 8
HandleIRQ_macro HandleIRQ_9, 9
HandleIRQ_macro HandleIRQ_10, 10
HandleIRQ_macro HandleIRQ_11, 11
HandleIRQ_macro HandleIRQ_12, 12
HandleIRQ_macro HandleIRQ_13, 13
HandleIRQ_macro HandleIRQ_14, 14
HandleIRQ_macro HandleIRQ_15, 15


	; if it is unidentified irq number, do nothing.
	global UnidentifiedIRQ
UnidentifiedIRQ:
	ret

no_schedule:
	RESTORE_ALL
	iret


