/********************************************************************
 * DESCRIPTION : functions and data for initializing Interrupt Descriptor Table
 * FILE NAME : IDT.h
 *******************************************************************/

#ifndef __IDT_H__
#define __IDT_H__


#include "GDT.h"


#define DESTYPE_INT	0x8e00	/* interrupt descriptor type : 01110 */
#define DESTYPE_TRAP	0x8f00	/* trap descriptor type : 01111 */

#define IDT_SIZE 256			/* length of table */

/* Interrupt Descriptor Table */
typedef struct _IDT {
	unsigned short OffsetLow;
	unsigned short SegSelector;
	unsigned short Type;
	unsigned short OffsetHigh;
} IDT_t;


/*
 * define IDT 
 */
IDT_t IDT[IDT_SIZE];



/* handlers of exceptions */
void HANDLER_DivideZero(void);
void HANDLER_Debug(void);
void HANDLER_NMI(void);
void HANDLER_BreakPoint(void);
void HANDLER_Overflow(void);
void HANDLER_BOUNDExceed(void);
void HANDLER_InvalidOpcode(void);
void HANDLER_DeviceDisable(void);
void HANDLER_DoubleFault(void);
void HANDLER_CoprocessorSegOver(void);
void HANDLER_InvalidTSS(void);
void HANDLER_SegNotPresent(void);
void HANDLER_StackFault(void);
void HANDLER_GeneralProtection(void);
void HANDLER_PageFault(void);
void HANDLER_INTELReserved(void);
void HANDLER_FloatingPoint(void);
void HANDLER_AlignCheck(void);
void HANDLER_MachineCheck(void);
void HANDLER_SIMDFault(void);

/* entry point of exceptions */
extern void DivideZero(void);
extern void Debug(void);
extern void NMI(void);
extern void BreakPoint(void);
extern void Overflow(void);
extern void BOUNDExceed(void);
extern void InvalidOpcode(void);
extern void DeviceDisable(void);
extern void DoubleFault(void);
extern void CoprocessorSegOver(void);
extern void InvalidTSS(void);
extern void SegNotPresent(void);
extern void StackFault(void);
extern void GeneralProtection(void);
extern void PageFault(void);
extern void INTELReserved(void);
extern void FloatingPoint(void);
extern void AlignCheck(void);
extern void MachineCheck(void);
extern void SIMDFault(void);

/* null handler */
extern void UnidentifiedINTR(void); /* declare in 'InitException.asm'*/

/* initialize IDT */
void SetIntrGate(int, void *);
void SetSystemGate(int, void *);
void SetTrapGate(int, void *);
void InitIDT(void);



/*
DivideZero
Debug
NMI
BreakPoint
Overflow
BOUNDExceed
InvalidOpcode
DeviceDisable
DoubleFault
CoprocessorSegOver
InvalidTSS
SegNotPresent
StackFault
GeneralProtection
PageFault
INTELReserved
FloatingPoint
AlignCheck
MachineCheck
SIMDFault
*/



#endif
